@@ -257,13 +257,12 @@ extract_text <- function(content) {
     cc <- content$content
     if (length(cc) == 0) return(NA_character_)
 
-    # Prefer JSON emitted via tool_use when structured mode is on
+    # Prefer tool_use when structured mode is on (unchanged)
     is_tool_use <- function(b) identical(b$type, "tool_use")
     has_tool <- any(vapply(cc, is_tool_use, logical(1)))
     if (has_tool) {
       idx <- which(vapply(cc, is_tool_use, logical(1)))[1]
       tu  <- cc[[idx]]
-      # tu$input is already a structured R list (from JSON); serialize to JSON string
       if (!is.null(tu$input)) {
         if (is.character(tu$input) && length(tu$input) == 1L) {
           return(tu$input)
@@ -273,13 +272,18 @@ extract_text <- function(content) {
       }
     }
 
-    # Fallback to plain text if no tool_use
-    if (!is.null(cc[[1]]$text)) return(cc[[1]]$text)
+    # pick the last text block, skipping the thinking block
+    text_blocks <- Filter(function(b) !is.null(b$text) && is.character(b$text) && nzchar(b$text[1]), cc)
+    if (length(text_blocks) > 0) {
+      return(text_blocks[[length(text_blocks)]]$text)
+    }
+
     return(NA_character_)
   }
 
 
 
+
   # Gemini (REST)
   if (!is.null(content$candidates)) {
     if (length(content$candidates) == 0) return(NA_character_)
@@ -388,7 +392,7 @@ get_endpoint <- function(config, default_endpoint) {
 #'
 #' @param provider Character scalar. One of:
 #'   `"openai"`, `"anthropic"`, `"gemini"`, `"groq"`, `"together"`,
-#'   `"voyage"` (embeddings only), `"deepseek"`, `"xai"`.
+#'   `"voyage"` (embeddings only), `"deepseek"`, `"xai"`, `"ollama"`.
 #' @param model Character scalar. Model name understood by the chosen provider.
 #'   (e.g., `"gpt-4o-mini"`, `"o4-mini"`, `"claude-3.7"`, `"gemini-2.0-flash"`, etc.)
 #' @param api_key Character scalar. Provider API key.
@@ -406,6 +410,9 @@ get_endpoint <- function(config, default_endpoint) {
 #' @param ... Additional provider-specific parameters (e.g., `temperature`,
 #'   `top_p`, `max_tokens`, `top_k`, `repetition_penalty`, `reasoning_effort`,
 #'   `api_url`, etc.). Values are forwarded verbatim unless documented shims apply.
+#'   For Anthropic extended thinking, supply `thinking_budget` (canonical;
+#'   mapped to `thinking.budget_tokens`) together with `include_thoughts = TRUE`
+#'   to request the thinking block in the response.
 #'
 #' @return An object of class `c("llm_config", provider)`. Fields:
 #'   `provider`, `model`, `api_key`, `troubleshooting`, `embedding`,
@@ -489,7 +496,7 @@ llm_config <- function(provider, model, api_key = NULL,
       # A literal token was supplied. Move it into a temporary env var and keep only its name.
       rand <- paste(sample(c(LETTERS, 0:9), 8, TRUE), collapse = "")
       env_name <- paste0("LLMR_", toupper(provider), "_KEY_", rand)
-     # Sys.setenv(structure(api_key, names = env_name)) ### not right
+      # Sys.setenv(structure(api_key, names = env_name)) ### not right
       do.call(Sys.setenv, setNames(list(api_key), env_name))
       api_key_handle <- llm_api_key_env(env_name)
       if (requireNamespace("cli", quietly = TRUE)) {
@@ -529,6 +536,8 @@ llm_config <- function(provider, model, api_key = NULL,
 
 
 
+ 
+
 #' Call an LLM (chat/completions or embeddings) with optional multimodal input
 #'
 #' `call_llm()` dispatches to the correct provider implementation based on
@@ -560,13 +569,18 @@ llm_config <- function(provider, model, api_key = NULL,
 #'         parameter as `max_tokens`, LLMR will, unless `no_change=TRUE`,
 #'         retry once replacing `max_tokens` with `max_completion_tokens`
 #'         (and inform via a `cli_alert_info`). The former experimental
-#'         “uncapped retry on empty content” is \emph{disabled} by default to
+#'         "uncapped retry on empty content" is \emph{disabled} by default to
 #'         avoid unexpected costs.
 #'   \item \strong{Anthropic:} `max_tokens` is required; if omitted LLMR uses
-#'         `2048` and warns. Multimodal images are inlined as base64.
+#'         `2048` and warns. Multimodal images are inlined as base64. Extended
+#'         thinking is supported: provide `thinking_budget` and
+#'         `include_thoughts = TRUE` to include a `content` block of type
+#'         `"thinking"` in the response; LLMR sets the beta header automatically.
 #'   \item \strong{Gemini (REST):} `systemInstruction` is supported; user
 #'         parts use `text`/`inlineData(mimeType,data)`; responses are set to
 #'         `responseMimeType = "text/plain"`.
+#'   \item \strong{Ollama (local):} OpenAI-compatible endpoints on `http://localhost:11434/v1/*`;
+#'         no Authorization header is required. Override with `api_url` as needed.
 #'   \item \strong{Error handling:} HTTP errors raise structured conditions with
 #'         classes like `llmr_api_param_error`, `llmr_api_rate_limit_error`,
 #'         `llmr_api_server_error`; see the condition fields for status, code,
@@ -574,8 +588,8 @@ llm_config <- function(provider, model, api_key = NULL,
 #' }
 #'
 #' @section Message normalization:
-#' See the \emph{“multimodal shortcut”} described under `messages`. Internally,
-#' LLMR expands these into the provider’s native request shape and tilde-expands
+#' See the \emph{"multimodal shortcut"} described under `messages`. Internally,
+#' LLMR expands these into the provider's native request shape and tilde-expands
 #' local file paths.
 #'
 #' @seealso
@@ -598,7 +612,16 @@ llm_config <- function(provider, model, api_key = NULL,
 #' as.character(r)
 #' finish_reason(r); tokens(r)
 #'
-#' ## 3) Multimodal (named-vector shortcut)
+#' ## 3) Anthropic extended thinking (single example)
+#' a_cfg <- llm_config("anthropic", "claude-sonnet-4-20250514",
+#'                     max_tokens = 5000,
+#'                     thinking_budget = 16000,
+#'                     include_thoughts = TRUE)
+#' r2 <- call_llm(a_cfg, "Compute 87*93 in your head. Give only the final number.")
+#' # thinking (if present): r2$raw$content[[1]]$thinking
+#' # final text:            r2$raw$content[[2]]$text
+#'
+#' ## 4) Multimodal (named-vector shortcut)
 #' msg <- c(
 #'   system = "Answer briefly.",
 #'   user   = "Describe this image in one sentence.",
@@ -606,13 +629,13 @@ llm_config <- function(provider, model, api_key = NULL,
 #' )
 #' call_llm(cfg, msg)
 #'
-#' ## 4) Embeddings
+#' ## 5) Embeddings
 #' e_cfg <- llm_config("voyage", "voyage-large-2",
 #'                     embedding = TRUE)
 #' emb_raw <- call_llm(e_cfg, c("first", "second"))
 #' emb_mat <- parse_embeddings(emb_raw)
 #'
-#' ## 5) With a chat session
+#' ## 6) With a chat session
 #' ch <- chat_session(cfg)
 #' ch$send("Say hello in Greek.")   # prints the same status line as `print.llmr_response`
 #' ch$history()
@@ -699,7 +722,7 @@ call_llm.openai <- function(config, messages, verbose = FALSE) {
       httr2::req_body_json(bdy)
   }
 
-  last_body <- NULL  # used for potential retries; keep for troubleshooting
+  last_body <- NULL  # used for potential retries and debugging
 
   run <- function(bdy) {
     last_body <<- bdy
@@ -723,7 +746,9 @@ call_llm.openai <- function(config, messages, verbose = FALSE) {
             sprintf("Replaced `max_tokens` with `max_completion_tokens` for %s after server hint.", config$model)
           )
         }
-        return(run(b2))
+        res2 <- run(b2)
+        last_body <<- b2
+        return(res2)
       }
       stop(e)
     }
@@ -754,76 +779,82 @@ call_llm.anthropic <- function(config, messages, verbose = FALSE) {
   messages <- .normalize_messages(messages)
   endpoint <- get_endpoint(config, default_endpoint = "https://api.anthropic.com/v1/messages")
 
-  use_thinking_beta <- !is.null(config$model_params$thinking_budget) ||
-    isTRUE(config$model_params$include_thoughts)  # kept for future beta headers
-
-  # Use the helper to separate system messages
+  # Separate system text and user/assistant messages
   formatted <- format_anthropic_messages(messages)
 
-  # Process user messages for multimodal content
+  # Build Anthropic message blocks:
+  # - content MUST be an array of blocks; never send a bare string
+  # - images are base64-inlined
   processed_user_messages <- lapply(formatted$user_messages, function(msg) {
-    # ── KEEP STRING CONTENT AS-IS ─────────────────────────────
-    if (is.character(msg$content)) {
-      return(list(role = msg$role, content = msg$content))
+    blocks <- if (is.character(msg$content)) {
+      list(list(type = "text", text = msg$content))
+    } else if (is.list(msg$content)) {
+      purrr::compact(lapply(msg$content, function(part) {
+        if (identical(part$type, "text")) {
+          list(type = "text", text = part$text)
+        } else if (identical(part$type, "file")) {
+          fd <- .process_file_content(part$path)
+          list(
+            type = "image",
+            source = list(
+              type = "base64",
+              media_type = fd$mime_type,
+              data = fd$base64_data
+            )
+          )
+        } else {
+          NULL
+        }
+      }))
+    } else {
+      list(list(type = "text", text = as.character(msg$content)))
     }
-
-    # ── OTHERWISE (images / tools) BUILD BLOCKS ───────────────
-    content_blocks <- lapply(msg$content, function(part) {
-      if (part$type == "text")
-        list(type = "text", text = part$text)
-      else if (part$type == "file") {
-        fd <- .process_file_content(part$path)
-        list(type = "image",
-             source = list(type = "base64",
-                           media_type = fd$mime_type,
-                           data = fd$base64_data))
-      } else NULL
-    })
-    list(role = msg$role, content = content_blocks |> purrr::compact())
+    list(
+      role = if (msg$role %in% c("user","assistant")) msg$role else "user",
+      content = blocks
+    )
   })
 
-  ### translate & pull out Anthropic-specific aliases
+  # Translate canonical params but keep unknowns
   params <- .translate_params("anthropic", config$model_params)
 
   if (is.null(params$max_tokens))
-    warning('Anthropic requires max_tokens; setting it at 2048.')
-
+    warning("Anthropic requires max_tokens; setting it at 2048.")
 
+  # thinking is enabled when either:
+  # - user sets thinking = TRUE (or "enabled"), or
+  # - a budget_tokens is provided directly (or via thinking_budget alias)
+  thinking_enabled <- isTRUE(config$model_params$thinking) ||
+    identical(tolower(as.character(config$model_params$thinking %||% "")), "enabled") ||
+    !is.null(params$budget_tokens)
 
   body <- .drop_null(list(
-    model      = config$model,
-    max_tokens = params$max_tokens %||% 2048,
-    temperature= params$temperature,
-    top_p      = params$top_p,
-    messages   = processed_user_messages,
-    thinking   = if (!is.null(params$thinking_budget) &&
-                     !is.null(params$include_thoughts))
-      list(
-        type          = "enabled",
-        budget_tokens = params$thinking_budget
-      )
+    model       = config$model,
+    max_tokens  = params$max_tokens %||% 2048,
+    temperature = params$temperature,
+    top_p       = params$top_p,
+    system      = formatted$system_text,                    # top-level system
+    messages    = processed_user_messages,
+    thinking    = if (thinking_enabled) .drop_null(list(
+      type          = "enabled",
+      budget_tokens = params$budget_tokens
+    )) else NULL
   ))
 
-  # Pass through any explicit tools/tool_choice already set by enable_structured_output() or user
+  # Respect tools/tool_choice if already present (e.g., from enable_structured_output)
   mp <- config$model_params %||% list()
   if (!is.null(mp$tools))       body$tools       <- c(body$tools %||% list(), mp$tools)
   if (!is.null(mp$tool_choice)) body$tool_choice <- mp$tool_choice
 
-
-
-  # Ensure Anthropic tool names are unique and tool_choice refers to an existing tool
+  # Ensure tool names unique and tool_choice valid, unchanged from your version
   if (!is.null(body$tools) && length(body$tools) > 0) {
-    # keep first occurrence of each name
     nm <- vapply(body$tools, function(t) t$name %||% "", character(1))
     keep <- !duplicated(nm) | nm == ""
     body$tools <- body$tools[keep]
-
-    # fix tool_choice if it points to a missing/empty name
     if (!is.null(body$tool_choice) && identical(body$tool_choice$type, "tool")) {
       want <- body$tool_choice$name %||% ""
       have <- vapply(body$tools, function(t) identical(t$name, want), logical(1))
       if (!nzchar(want) || !any(have)) {
-        # default to first named tool
         first_named <- which(nzchar(vapply(body$tools, function(t) t$name %||% "", character(1))))[1]
         if (!is.na(first_named)) {
           body$tool_choice <- list(type = "tool", name = body$tools[[first_named]]$name)
@@ -839,10 +870,7 @@ call_llm.anthropic <- function(config, messages, verbose = FALSE) {
       "Content-Type"      = "application/json",
       "x-api-key"         = .resolve_api_key(config$api_key, provider = config$provider, model = config$model),
       "anthropic-version" = "2023-06-01",
-      !!! (
-        if (!is.null(body$thinking))
-          list("anthropic-beta" = "extended-thinking-2025-05-14")
-      )
+      !!! ( if (!is.null(mp$anthropic_beta)) list("anthropic-beta" = mp$anthropic_beta) else NULL )
     ) |>
     httr2::req_body_json(body)
 
@@ -1162,6 +1190,70 @@ call_llm.xai <- function(config, messages, verbose = FALSE) {
 
 
 
+#' @export
+#' @rdname call_llm
+#' @section Using a local Ollama server:
+#' Ollama provides an OpenAI-compatible HTTP API on localhost by default. Start the
+#' daemon and pull a model first (terminal): `ollama serve` (in background) and
+#' `ollama pull llama3`. Then configure LLMR with
+#' `llm_config("ollama", "llama3", embedding = FALSE)` for chat or
+#' `llm_config("ollama", "nomic-embed-text", embedding = TRUE)` for embeddings.
+#' Override the endpoint with `api_url` if not using the default
+#' `http://localhost:11434/v1/*`.
+call_llm.ollama <- function(config, messages, verbose = FALSE) {
+  if (isTRUE(config$embedding) ||
+      grepl("embedding", config$model, ignore.case = TRUE)) {
+    return(call_llm.ollama_embedding(config, messages, verbose))
+  }
+  messages <- .normalize_messages(messages)
+  endpoint <- get_endpoint(config, default_endpoint = "http://localhost:11434/v1/chat/completions")
+
+  body <- .drop_null(list(
+    model       = config$model,
+    messages    = messages,
+    temperature = config$model_params$temperature,
+    top_p       = config$model_params$top_p,
+    max_tokens  = config$model_params$max_tokens
+  ))
+
+  # Structured outputs (OpenAI-compatible)
+  mp <- config$model_params %||% list()
+  if (!is.null(mp$response_format)) body$response_format <- mp$response_format
+  if (is.null(body$response_format) && !is.null(mp$json_schema)) {
+    body$response_format <- list(
+      type = "json_schema",
+      json_schema = list(name = "llmr_schema", schema = mp$json_schema, strict = TRUE)
+    )
+  }
+  if (!is.null(mp$tools))       body$tools       <- mp$tools
+  if (!is.null(mp$tool_choice)) body$tool_choice <- mp$tool_choice
+
+  req <- httr2::request(endpoint) |>
+    httr2::req_headers(
+      "Content-Type" = "application/json"
+    ) |>
+    httr2::req_body_json(body)
+
+  perform_request(req, verbose, provider = config$provider, model = config$model)
+}
+
+#' @export
+#' @keywords internal
+call_llm.ollama_embedding <- function(config, messages, verbose = FALSE) {
+  endpoint <- get_endpoint(config, default_endpoint = "http://localhost:11434/v1/embeddings")
+  texts <- if (is.character(messages)) messages else vapply(messages, `[[`, "", "content")
+  body <- list(model = config$model, input = texts)
+
+  extras <- config$model_params[setdiff(names(config$model_params), names(body))]
+  body   <- .drop_null(c(body, extras))
+
+  req <- httr2::request(endpoint) |>
+    httr2::req_headers("Content-Type" = "application/json") |>
+    httr2::req_body_json(body)
+
+  perform_request(req, verbose, provider = config$provider, model = config$model)
+}
+
 # ----- Embedding-specific Handlers -----
 
 #' @export
@@ -1383,6 +1475,7 @@ bind_tools <- function(config, tools, tool_choice = NULL) {
 #' @param verbose Logical. If TRUE, prints progress messages. Default is TRUE.
 #'
 #' @return A numeric matrix where each row is an embedding vector for the corresponding text.
+#'   Columns are named \code{v1}, \code{v2}, ..., \code{vK} where K is the embedding dimension.
 #'   If embedding fails for certain texts, those rows will be filled with NA values.
 #'   The matrix will always have the same number of rows as the input texts.
 #'   Returns NULL if no embeddings were successfully generated.
@@ -1493,10 +1586,13 @@ get_batched_embeddings <- function(texts,
   # Combine all embeddings into final matrix
   final_embeddings <- do.call(rbind, emb_list)
 
-  if(!is.null(names(texts))){
+  if (!is.null(names(texts))) {
     rownames(final_embeddings) <- names(texts)
   }
 
+  # Always assign stable column names v1..vK for downstream consistency
+  colnames(final_embeddings) <- paste0("v", seq_len(ncol(final_embeddings)))
+
   if (verbose) {
     n_successful <- sum(stats::complete.cases(final_embeddings))
     message("Successfully generated embeddings for ", n_successful,
